#include "Engine.h"

void Engine::start_game(const int max_rnds)
{
    human_choice = computer_choice = Choice::NONE;
    human_wins = computer_wins = ties = 0;

    round = 0;
    max_rounds = max_rnds;

    display->show_game_start();
}

void Engine::end_game()
{
    display->show_game_end(human_wins, computer_wins, ties, max_rounds);
}

void Engine::next_round()
{
    round++;
    if (round <= max_rounds)
    {
        display->show_round(round);
    }
    else
    {
        end_game();
    }
}

void Engine::do_rounds()
{
    while (round <= max_rounds)
    {
        do_round();
        next_round();
    }
}

void Engine::do_round()
{
    human_choice     = human->choose();
    computer_choice  = computer->choose();
    predicted_choice = computer->get_prediction();

    winner = determine_winner(human_choice, computer_choice);

    display->update(human_choice, computer_choice, predicted_choice,
                    winner, human_wins, computer_wins, ties);
}

Who Engine::determine_winner(const Choice human_choice,
                             const Choice computer_choice)
{
    Who winner;

    if (human_choice > computer_choice)
    {
        winner = Who::HUMAN;
        human_wins++;
    }
    else if (computer_choice > human_choice)
    {
        winner = Who::COMPUTER;
        computer_wins++;
    }
    else
    {
        winner = Who::TIE;
        ties++;
    }

    return winner;
}
