#include <stdlib.h>
#include <time.h>

#include "AlgorithmFactory.h"
#include "Player.h"
#include "Human.h"
#include "Computer.h"
#include "Who.h"
#include "TextDisplay.h"
#include "Engine.h"
#include "RPS.h"

RPS::RPS(const AlgorithmType computer_algorithm_type, const int max_rounds)
{
    // Seed the random number generator.
    srand(time(0));

    // Players.
    Human    *human    = new Human(Who::HUMAN);
    Computer *computer = new Computer(Who::COMPUTER);

    // Display and engine.
    TextDisplay *text_display = new TextDisplay();
    Engine *engine = new Engine(human, computer, text_display);

    // Choice algorithms.
    Algorithm *human_algorithm =
                        AlgorithmFactory::make(AlgorithmType::PROMPT);
    Algorithm *computer_algorithm =
                        AlgorithmFactory::make(computer_algorithm_type);

    human_algorithm->set_engine(engine);
    computer_algorithm->set_engine(engine);

    human->set_algorithm(human_algorithm);
    computer->set_algorithm(computer_algorithm);

    // Play a game.
    engine->start_game(max_rounds);
    engine->next_round();
    engine->do_rounds();

    delete human_algorithm;
    delete computer_algorithm;
    delete text_display;
    delete engine;
    delete human;
    delete computer;
}
