#ifndef SMARTALGORITHM_H_
#define SMARTALGORITHM_H_

#include <string>
#include <map>

#include "RandomAlgorithm.h"
#include "Algorithm.h"
#include "Choice.h"
#include "Who.h"

using namespace std;

const string FREQUENCY_FILE_NAME = "freq.txt";
const int    PATTERN_LENGTH      = 5;

class SmartAlgorithm : public Algorithm
{
public:
    SmartAlgorithm();
    virtual ~SmartAlgorithm();

private:
    int round;                          // current round
    int patternLength;                  // length of search pattern string
    string history;                     // recent history of players' choices
    string filename;                    // frequency file name
    map<string, int> frequencies;       // frequencies of history strings
                                        //   to predict the human's next choice
    RandomAlgorithm *random_algorithm;  // for when no prediction is possible

    /**
     * Make a prediction based on the choice history.
     * @return the predicted choice.
     */
    virtual Choice predict() override;

    /**
     * Decide on a choice depending on the prediction.
     * @param prediction the prediction.
     * @return the decided choice.
     */
    virtual Choice decide(const Choice prediction) override;

    /**
     * Check the frequency of the given history string.
     * @param history the history string.
     * @return the frequency.
     */
    int check_frequency(const string history) const;

    /**
     * Update the frequency of the history string.
     * @param human_choice the human's choice
     * @param computer_choice the computer's choice
     */
    void update_frequency(const Choice human_choice,
                          const Choice computer_choice);

    /**
     * Append to the recent history, up to MAX_HISTORY length.
     * @param obj a chosen object.
     */
    void append_history(const Who who, const Choice choice);

    /**
     * Read the frequencies file.
     */
    void read_frequencies();

    /**
     * Write the frequencies file.
     */
    void write_frequencies();
};

#endif /* SMARTALGORITHM_H_ */
